#ifndef _UTIL_H
#define _UTIL_H

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-variable"

#include <math.h>
#include <stdint.h>

#include <alleycat.h>

#define err_str_size 200
static char drup_util_err_str[err_str_size];

#define try_with(f, args, onfail) do {\
  int rc = snprintf (drup_util_err_str, err_str_size, "%s (): failed at %s ()", __func__, #f); \
  if (rc >= err_str_size) { \
    drup_util_err_str[err_str_size - 1] = '.'; \
    drup_util_err_str[err_str_size - 2] = '.'; \
    drup_util_err_str[err_str_size - 3] = '.'; \
  } \
  if (!f args) onfail (drup_util_err_str); \
} while (0);

/* The main form we use shows errors with `perr` and returns `false`.
 */
#define try(args...) do {\
  try_with (args, ac_warn_perr_rf); \
} while (0);

#define try_with_info(f, args, onfail, info) do {\
  int rc = snprintf (drup_util_err_str, err_str_size, "%s (): failed at %s () %s", __func__, #f, #info); \
  if (rc >= err_str_size) { \
    drup_util_err_str[err_str_size - 1] = '.'; \
    drup_util_err_str[err_str_size - 2] = '.'; \
    drup_util_err_str[err_str_size - 3] = '.'; \
  } \
  if (!f args) onfail (drup_util_err_str); \
} while (0);

#define try_info(f, args, info) do {\
  try_with_info(f, args, ac_warn_perr_rf, info); \
} while (0);

void uint64_to_hex (uint64_t value, char ret[]);
void hex_to_uint64 (const char *value, uint64_t *ret);
uint8_t calculate_config (uint8_t val);
void secret_16_to_8 (const uint8_t *secret16, uint8_t *secret8);
bool roundu8 (double f, uint8_t *u);
bool get_bit (uint8_t u, uint8_t mask);

#pragma GCC diagnostic pop

#endif
