#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include <alleycat.h>

#include "private/util.h"

/* `ret`: uint8_t[17]
 */
void uint64_to_hex (uint64_t const value, char ret[]) {
  for (int i = 0; i < 8; i++) {
     uint8_t byte = (uint8_t) (value >> i*8);
     sprintf (ret + i*2, "%02X", byte);
  }
}

/* `value`: uint8_t[17]
 */
void hex_to_uint64 (const char * const value, uint64_t * const ret) {
  uint64_t result = 0;
  for (int i = 0; i < 8; i++) {
    int j = i * 2;
    char hex[3];
    strncpy (hex, value + j, 2);
    hex[2] = '\0';
    uint8_t byte = strtol (hex, NULL, 16);
    result |= (uint64_t) byte << i*8;
  }
  *ret = result;
}

/*
 * | bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2 | bit 1 | bit 0
 *   ------ ------- ------- ------- ------- ------- ------- ------
 *    ---     ---             ---
 *    1WS     SPU      1      APU     1WS     SPU      0      APU
 */

// @todo bitmask with 0x0f doesn't seem necessary.
uint8_t calculate_config (uint8_t const val) {
  return val | ((~val & 0x0f) << 4);
}

/* `secret16`: 16 * uint8_t (what we get passed via the public api)
 * `secret8`: 8 * uint8_t
 */
void secret_16_to_8 (const uint8_t * const secret16, uint8_t * const secret8) {
  for (int i = 0; i < 8; i++) {
    secret8[i] = (secret16[2 * i + 1] << 8) | (secret16[2 * i]);
  }
}

/* `f` must be in the range [0.0, 255.0]
 */
bool roundu8 (double f, uint8_t *u) {
  if (f < 0.0 || f > 255.0) {
    ac_complain (__FILE__, __LINE__, true, false, "roundu8: arg must be between [0.0, 255.0]");
    return false;
  }
  double f0 = trunc (f);
  double fd = f - f0;
  if (fd >= 0.5) *u = (uint8_t) f + 1;
  else *u = (uint8_t) f;
  return true;
}

bool get_bit (uint8_t u, uint8_t mask) {
  uint8_t shift = 0;
  double shiftd = log2 (mask);
  // --- can't fail: shiftd is definitely within the bounds.
  roundu8 (shiftd, &shift);
  return (u & mask) >> shift;
}
